/* Include required headers */

#include "MathLink.h"
#include "WolframLibrary.h"
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <sys/stat.h>
#include <time.h>

static char accessStr[50];
static char permStr[10];

/* Return the version of Library Link */
EXTERN_C DLLEXPORT mint WolframLibrary_getVersion()
{
	return WolframLibraryVersion;
}

/* Initialize Library */
EXTERN_C DLLEXPORT int WolframLibrary_initialize(WolframLibraryData libData)
{
	return LIBRARY_NO_ERROR;
}

/* Uninitialize Library */
EXTERN_C DLLEXPORT void WolframLibrary_uninitialize(WolframLibraryData libData)
{
	return;
}

/* Input: string representing file path
 * Output: int with file size in bytes
 */
EXTERN_C DLLEXPORT int myFileSize(WolframLibraryData libData, mint Argc, MArgument *Args, MArgument res)
{
	struct stat statbuf;
	mint bytesize = 0;
	char *filepath = MArgument_getUTF8String(Args[0]);
	int errcode;
	
	errcode = stat(filepath, &statbuf);
	libData->UTF8String_disown(filepath);
	if (errcode != 0)
		return LIBRARY_FUNCTION_ERROR;
	
	bytesize = statbuf.st_size;
	MArgument_setInteger(res, bytesize);
	return LIBRARY_NO_ERROR;
}

/* Input: string representing file path
 * Output: string stating the date and time
 */
EXTERN_C DLLEXPORT int myFileAccessTime(WolframLibraryData libData, mint Argc, MArgument *Args, MArgument res)
{
	struct stat statbuf;
	mint bytesize = 0;
	char *filepath = MArgument_getUTF8String(Args[0]);
	int errcode;
	char *str;
	
	errcode = stat(filepath, &statbuf);
	libData->UTF8String_disown(filepath);
	if (errcode != 0)
		return LIBRARY_FUNCTION_ERROR;
	
	str = ctime_r(&statbuf.st_atimespec.tv_sec, accessStr);
	if (str == NULL)
		return LIBRARY_FUNCTION_ERROR;
	
	MArgument_setUTF8String(res, str);
	return LIBRARY_NO_ERROR;
}

/* Input: string representing file path
 * Output: int with file size in bytes
 */
EXTERN_C DLLEXPORT int myFilePermissions(WolframLibraryData libData, mint Argc, MArgument *Args, MArgument res)
{
	struct stat statbuf;
	mint bytesize = 0;
	char *filepath = MArgument_getUTF8String(Args[0]);
	int errcode;
	char *str;
	mode_t mode;
	
	errcode = stat(filepath, &statbuf);
	libData->UTF8String_disown(filepath);
	if (errcode != 0)
		return LIBRARY_FUNCTION_ERROR;
	
	mode = statbuf.st_mode;
	if (mode & S_IRUSR)
		permStr[0] = 'r';
	else
		permStr[0] = '-';
	if (mode & S_IWUSR)
		permStr[1] = 'w';
	else
		permStr[1] = '-';
	if (mode & S_IXUSR)
		permStr[2] = 'x';
	else
		permStr[2] = '-';
	if (mode & S_IRGRP)
		permStr[3] = 'r';
	else
		permStr[3] = '-';
	if (mode & S_IWGRP)
		permStr[4] = 'w';
	else
		permStr[4] = '-';
	if (mode & S_IXGRP)
		permStr[5] = 'x';
	else
		permStr[5] = '-';
	if (mode & S_IROTH)
		permStr[6] = 'r';
	else
		permStr[6] = '-';
	if (mode & S_IWOTH)
		permStr[7] = 'w';
	else
		permStr[7] = '-';
	if (mode & S_IXOTH)
		permStr[8] = 'x';
	else
		permStr[8] = '-';
	permStr[9] = 0;
	
	MArgument_setUTF8String(res, permStr);
	return LIBRARY_NO_ERROR;
}

/* Input: string representing file path
 * Output: int with file size in bytes
 */
/*
EXTERN_C DLLEXPORT int myFileExtendedAttributes(WolframLibraryData libData, MLINK mlp)
{
	struct stat statbuf;
	const unsigned char* filepath;
	int errcode, bytes, characters, len = 0;
	
	if (!MLCheckFunction(mlp, "List", &len) || len != 1)
		return LIBRARY_FUNCTION_ERROR;
	
	if (!MLGetUTF8String(mlp, &filepath, &bytes, &characters))
		return LIBRARY_FUNCTION_ERROR;
	
/*	errcode = stat(filepath, &statbuf);
	MLReleaseUTF8String(mlp, filepath, bytes);
	if (errcode != 0)
		return LIBRARY_FUNCTION_ERROR;*/
/*
	if (!MLNewPacket(mlp))
		return LIBRARY_FUNCTION_ERROR;

	if (!MLPutFunction(mlp, "List", 1))
		return LIBRARY_FUNCTION_ERROR;

	if (!MLPutUTF8String(mlp, "Test", 4))
		return LIBRARY_FUNCTION_ERROR;
	
	if (!MLEndPacket(mlp))
		return LIBRARY_FUNCTION_ERROR;

	if (!MLFlush(mlp))
		return LIBRARY_FUNCTION_ERROR;
	*/
	return LIBRARY_NO_ERROR;
}
*/

